{-----------------------------------------------------------------------
*  TSlideBar v1.1                                                      *
*                                                                      *
*    Copyright  1996, 1997 InforTech, Inc.  All Rights Reserved.      *
*    Developed By: Steven Costa Martins                                *
*    E-MAIL: steven.martins@workmail.com                               *
						 infortech@usa.net                                         *
*    Home Page:                                                        *
*    http://www.geocities.com/SiliconValley/Park/7619/                 *
*                                                                      *
*    Last Modification: 23 Dec 1996, 10:21 AM                          *
------------------------------------------------------------------------
*  You are not allowed to redistribute this                            *
*  code or any part of it                                              *
------------------------------------------------------------------------
* Changes by Davide Moretti <dmoretti@iper.net>  (marked with *>)      *
* Changes by Anthony Steele <asteele@aztec.co.za>  (marked with AFS)   *
*                                                                      *
* 20 Oct 1996                                                          *
*   Merged the 16bit and 32bit code into a single file                 *
*   Removed the HandCursor bug                                         *
*   Changed reals to singles (reals are not Intel-native so are slower)*
* December 1996                                                        *
*   Added new thumb types that look like win'95 Trackbar               *
*   Added the Frequency, LineSize and PageSize properties              *
*   (like the Win'95 Trackbar)                                         *
*                                                                      *
------------------------------------------------------------------------}
unit SlideBar;

interface

{$R SBIcons.RES}

{$A+} { Word align data }
{$B-} { Complete Boolean Evaluation Directive }
{$D-} { Debug Information Directive }
{$L-} { Local Symbol Information Directive }
{$R-} { Range checking off }
{$S-} { Stack checking off }
{$T-} { We don't need (nor want) this type checking! }

uses
  {$IFDEF WIN32}
    Windows,
  {$ELSE}
    WinTypes, WinProcs,
  {$ENDIF}
  SysUtils, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, Menus, DsgnIntf;

type
  (*
   *  Property AboutBox
   *)
  TAboutSlideBarProperty = class(TPropertyEditor)
  public
    procedure Edit; override;
    function GetAttributes: TPropertyAttributes; override;
    function GetValue: string; override;
  end;
  {------------------------------------------------------------------------}
  TBarStyle    = (bsLowered, bsRaised);
  TOrientation = (orVertical, orHorizontal);
  TThumbStyle  = (tsBar1, tsBar2, tsBar3, tsBar4, tsCircle1, tsSquare1,
								  tsDiamond1, tsDiamond2, tsDiamond3, tsDiamond4,
                   {AFS} tsStdDown, tsStdUp, tsStdVBar, tsStdUpDown,
                   	tsStdLeft, tsStdRight, tsStdHBar, stStdLeftRight);

  TSlideBar = class(TCustomControl)
  private
		FAbout									: TAboutSlideBarProperty;
    FFocusColor             : TColor;
    FHandCursor             : Boolean;
    FLabels                 : TStringList;
    FMax,FMin,FPosition     : Integer;
    FOrientation            : TOrientation;
    FStyle                  : TBarStyle;
    FThickness              : Byte;
    FThumbStyle             : TThumbStyle;
    FTicks                  : Boolean;

    { AFS }
    FFrequency							: integer;
    FLineSize							  : integer;
    FPageSize					      : integer;

    FOnChange               : TNotifyEvent;
    ThumbBmp,MaskBmp,BkgdBmp: TBitmap;
    DragVal,HalfTW,HalfTH   : Integer;
    ThumbRect               : TRect;
    TempDC                  : HDC;
    HandPointer             : HCursor;
    procedure SetLabels(Value: TStringList);
    procedure SetMax(Value: Integer);
    procedure SetMin(Value: Integer);
    procedure SetOrientation(Value: TOrientation);
    procedure SetPosition(Value: Integer);
    procedure SetStyle(Value: TBarStyle);
    procedure SetThickness(Value: Byte);
    procedure SetThumbStyle(Value: TThumbStyle);
    procedure SetTicks(Value: Boolean);

    { AFS }
    procedure SetFrequency (Value: integer);
    procedure SetLineSize (Value: integer);
    procedure SetPageSize (Value: integer);

    procedure CMEnter(var Message: TCMGotFocus); message CM_ENTER;
    procedure CMExit(var Message: TCMExit); message CM_EXIT;
    procedure WMGetDlgCode(var Message: TWMGetDlgCode); message WM_GETDLGCODE;
    procedure WMSetCursor(var Message: TWMSetCursor); message WM_SETCURSOR;
    procedure WMSize(var Message: TWMSize); message WM_SIZE;
  protected
    Dragging: Boolean;
    procedure Paint; override;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    function  NewPosition(WhereX,WhereY: Integer): Integer;
    function  IsVert: Boolean;
    procedure RemoveThumbBar;
    procedure DrawThumbBar;
    procedure DrawTrench;
    procedure SaveBackground;
    procedure WhereIsBar;
    procedure SetTLColor;
    procedure SetBRColor;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function CurrentLabel: String;

    { AFS }
    procedure MoveDown (iDist: integer);
		procedure MoveUp (iDist: integer);

  published
    property About: TAboutSlideBarProperty read FAbout write FAbout;
    property Enabled;
    property FocusColor: TColor read FFocusColor write FFocusColor default clBlack;
    property HandCursor: Boolean read FHandCursor write FHandCursor default True;
    property Labels: TStringList read FLabels write SetLabels;
    property Max: Integer read FMax write SetMax default 10;
    property Min: Integer read FMin write SetMin default 1;
    property Orientation: TOrientation read FOrientation write SetOrientation default orHorizontal;
    property ParentShowHint;
    property Position: Integer read FPosition write SetPosition default 1;
    property PopupMenu;
    property ShowHint;
    property Style: TBarStyle read FStyle write SetStyle default bsLowered;
    property TabStop default True;
    property TabOrder;
    property Thickness: Byte read FThickness write SetThickness default 1;
    property ThumbStyle: TThumbStyle read FThumbStyle write SetThumbStyle default tsCircle1;
    property Ticks: Boolean read FTicks write SetTicks default True;

    { AFS }
    property Frequency : integer read FFrequency write SetFrequency default 1;
    property LineSize	: integer read FLineSize write SetLineSize default 1;
    property PageSize	: integer read FPageSize write SetPageSize;

    property Visible;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
  end;

procedure Register;

implementation


function MinInt(A, B: Integer): Integer;
begin
  If A > B Then MinInt := B Else MinInt := A;
end;

function MaxInt(A,B: Integer): Integer;
begin
  If A > B Then MaxInt := A Else MaxInt := B;
end;

(*
 * TSlideBar Methods
 *)

constructor TSlideBar.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Height := 15;
  Width := 100;
  ThumbBmp := TBitmap.Create;
  MaskBmp := TBitmap.Create;
  BkgdBmp := TBitmap.Create;
  HandPointer := LoadCursor(HInstance, 'HandPointer');
  FFocusColor := clBlack;
  FHandCursor := True;
  FLabels := TStringList.Create;
  FMin := 1;
  FMax := 10;
  FOrientation := orHorizontal;
  FPosition := 1;
  FStyle := bsLowered;
  FThickness := 1;
  FTicks := True;
  { AFS }
  FFrequency := 1;
  FLineSize := 1;
  FPageSize := 10;
  Dragging := False;
  DragVal := 0;
  ThumbStyle := tsCircle1;
  TabStop := True;
end;

destructor TSlideBar.Destroy;
begin
  FLabels.Free;
  ThumbBmp.Free;
  MaskBmp.Free;
  BkgdBmp.Free;
  inherited Destroy;
end;

procedure TSlideBar.CMEnter(var Message: TCMGotFocus);
begin
  inherited;
  Refresh;
end;

procedure TSlideBar.CMExit(var Message: TCMExit);
begin
  inherited;
  Refresh;
end;

function TSlideBar.IsVert: Boolean;
begin
  IsVert := (Orientation = orVertical);
end;

procedure TSlideBar.KeyDown (var Key: Word; Shift: TShiftState);
begin

	{ AFS - Factored in PageSize and LineSize
   created MoveUp & MoveDown public procedures }
	case Key of
    VK_PRIOR : MoveDown (PageSize);
    VK_NEXT  : MoveUp (PageSize);
    VK_END   : if IsVert then Position := Min else Position := Max;
    VK_HOME  : if IsVert then Position := Max else Position := Min;
    VK_LEFT	 : MoveDown (LineSize);
    VK_DOWN  : MoveDown (LineSize);
    VK_UP    : MoveUp (LineSize);
    VK_RIGHT : MoveUp (LineSize);
  end; { Case }

end; { KeyDown }

procedure TSlideBar.MoveDown (iDist: integer);
begin
	if iDist = 0 then
  	exit;

	if iDist < 0 then
  begin
  	MoveUp (-iDist);
    exit;
  end;

  if (Position - iDist) > Min then
    Position := Position - iDist
  else
    Position := Min;
end;

procedure TSlideBar.MoveUp (iDist: integer);
begin
	if iDist = 0 then
  	exit;

	if iDist < 0 then
  begin
  	MoveDown (-iDist);
    exit;
  end;

  if (Position + iDist) < Max then
    Position := Position + iDist
  else
    Position := Max;
end;

procedure TSlideBar.WMGetDlgCode(var Message: TWMGetDlgCode);
begin
  Message.Result := DLGC_WANTARROWS;
end;

procedure TSlideBar.WMSetCursor(var Message: TWMSetCursor);
  begin
  if (not (csDesigning in ComponentState)) and HandCursor then
    {$IFDEF WIN32}
    Windows.SetCursor(HandPointer)
	  {$ELSE}
    SetCursor(HandPointer)
    {$ENDIF}
  else
    inherited;
  end;

procedure TSlideBar.WMSize(var Message: TWMSize);
begin
  if Height > Width then
    Orientation := orVertical else Orientation := orHorizontal;
end;

procedure TSlideBar.SetLabels(Value: TStringList);
begin
  FLabels.Assign(Value);
end;

procedure TSlideBar.SetMin(Value: Integer);
begin
  FMin := Value;
  { AFS - set page size to 1/10 of the range }
  PageSize := MaxInt(1,(Max-Min) div 10);
  Refresh;
end;

procedure TSlideBar.SetMax(Value: Integer);
begin
  FMax := Value;
  { AFS - set page size to 1/10 of the range }
  PageSize := MaxInt(1,(Max-Min) div 10);
  Refresh;
end;

procedure TSlideBar.SetOrientation(Value: TOrientation);
begin
  FOrientation := Value;
  Refresh;
end;

procedure TSlideBar.SetPosition(Value: Integer);
begin
  if csDesigning in ComponentState then
    begin
      if (Value >= Min) and (Value <= Max) Then FPosition := Value;
      Refresh;
    end
  else
    begin
      RemoveThumbBar;
      if (Value >= Min) and (Value <= Max) Then FPosition := Value;
      WhereIsBar;
      SaveBackground;
      DrawThumbBar;
      if Assigned(FOnChange) then FOnChange(Self);
    end;
end;

procedure TSlideBar.SetStyle(Value: TBarStyle);
begin
  FStyle := Value;
  Refresh;
end;

procedure TSlideBar.SetThickness(Value: Byte);
begin
  If (Value > 0) and (Value < 6) then
  begin
    FThickness := Value;
    Refresh;
  end;
end;

procedure TSlideBar.SetThumbStyle(Value: TThumbStyle);
begin
  If ThumbStyle <> Value then
    begin
      FThumbStyle := Value;
      case ThumbStyle of
				tsBar1     :   ThumbBmp.Handle := LoadBitmap(HInstance,'Bar1');
				tsBar2     :   ThumbBmp.Handle := LoadBitmap(HInstance,'Bar2');
				tsBar3     :   ThumbBmp.Handle := LoadBitmap(HInstance,'Bar3');
				tsBar4     :   ThumbBmp.Handle := LoadBitmap(HInstance,'Bar4');
				tsCircle1  :   ThumbBmp.Handle := LoadBitmap(HInstance,'Circle1');
				tsSquare1  :   ThumbBmp.Handle := LoadBitmap(HInstance,'Square1');
				tsDiamond1 :   ThumbBmp.Handle := LoadBitmap(HInstance,'Diamond1');
				tsDiamond2 :   ThumbBmp.Handle := LoadBitmap(HInstance,'Diamond2');
				tsDiamond3 :   ThumbBmp.Handle := LoadBitmap(HInstance,'Diamond3');
				tsDiamond4 :   ThumbBmp.Handle := LoadBitmap(HInstance,'Diamond4');

        { AFS }
        tsStdDown  :	 ThumbBmp.Handle := LoadBitmap(HInstance,'StdDown');
        tsStdUp  	 :	 ThumbBmp.Handle := LoadBitmap(HInstance,'StdUp');
        tsStdVBar  :	 ThumbBmp.Handle := LoadBitmap(HInstance,'StdVBar');
        tsStdUpDown:	 ThumbBmp.Handle := LoadBitmap(HInstance,'StdUpDown');
        tsStdLeft  :   ThumbBmp.Handle := LoadBitmap(HInstance,'StdLeft');
        tsStdRight :   ThumbBmp.Handle := LoadBitmap(HInstance,'StdRight');
        tsStdHBar  :   ThumbBmp.Handle := LoadBitmap(HInstance,'StdHBar');
        stStdLeftRight:ThumbBmp.Handle := LoadBitmap(HInstance,'StdLeftRight');
				else					 Raise Exception.Create ('TSlideBar.SetThumbStyle: Bad value');

      end; { case }

      case ThumbStyle of
				tsBar1     :   MaskBmp.Handle := LoadBitmap(HInstance,'Bar1Mask');
				tsBar2     :   MaskBmp.Handle := LoadBitmap(HInstance,'Bar2Mask');
				tsBar3     :   MaskBmp.Handle := LoadBitmap(HInstance,'Bar3Mask');
				tsBar4     :   MaskBmp.Handle := LoadBitmap(HInstance,'Bar4Mask');
				tsCircle1  :   MaskBmp.Handle := LoadBitmap(HInstance,'Circle1Mask');
				tsSquare1  :   MaskBmp.Handle := LoadBitmap(HInstance,'Square1Mask');
				tsDiamond1 :   MaskBmp.Handle := LoadBitmap(HInstance,'Diamond1Mask');
				tsDiamond2 :   MaskBmp.Handle := LoadBitmap(HInstance,'Diamond2Mask');
				tsDiamond3 :   MaskBmp.Handle := LoadBitmap(HInstance,'Diamond3Mask');
				tsDiamond4 :   MaskBmp.Handle := LoadBitmap(HInstance,'Diamond4Mask');
        { AFS }
        tsStdDown  :	 MaskBmp.Handle := LoadBitmap(HInstance,'StdDownMask');
        tsStdUp    :	 MaskBmp.Handle := LoadBitmap(HInstance,'StdUpMask');
        tsStdVBar   :	 MaskBmp.Handle := LoadBitmap(HInstance,'StdVBarMask');
        tsStdUpDown:	 MaskBmp.Handle := LoadBitmap(HInstance,'StdUpDownMask');
	      tsStdLeft  :   MaskBmp.Handle := LoadBitmap(HInstance,'StdLeftMask');
        tsStdRight :   MaskBmp.Handle := LoadBitmap(HInstance,'StdRightMask');
        tsStdHBar  :   MaskBmp.Handle := LoadBitmap(HInstance,'StdHBarMask');
        stStdLeftRight:MaskBmp.Handle := LoadBitmap(HInstance,'StdLeftRightMask');
				else					 Raise Exception.Create ('TSlideBar.SetThumbStyle: Bad value');
      end; { case }

      HalfTH := ThumbBmp.Height div 2;
      HalfTW := ThumbBmp.Width div 2;
      Refresh;
    end;
end;

procedure TSlideBar.SetTicks(Value: Boolean);
begin
	if FTicks = Value then
  	exit; { AFS}
  FTicks := Value;
  Refresh;
end;

{ AFS }
procedure TSlideBar.SetFrequency (Value: integer);
begin
	if FFrequency = Value then
  	exit;
  FFrequency := Value;
  Refresh;
end; {procedure SetFrequency }

procedure TSlideBar.SetLineSize (Value: integer);
begin
	if FLineSize = Value then
  	exit;
  if (Value < 1) or (Value > (Max - Min)) then
  	exit;

  FLineSize := Value;

  if PageSize < LineSize then
  	FPageSize := LineSize;
end; {procedure SetLineSize }
                                             
procedure TSlideBar.SetPageSize (Value: integer);
begin
	if FPageSize = Value then
  	exit;
  if (Value < 1) or (Value > (Max - Min)) then
  	exit;

  FPageSize := Value;

  if LineSize > PageSize then
  	FLineSize := PageSize;
end; { procedure SetPageSize }


function TSlideBar.CurrentLabel: String;
begin
  if ((Position-Min+1) <= Labels.Count) and (Position >= Min) then
    CurrentLabel := Labels[Position-Min]
  else
    CurrentLabel := '<Un-Defined>';
end;

function TSlideBar.NewPosition(WhereX,WhereY: Integer): Integer;
var
  H1,W1 : Integer;
begin
  {Calculate the nearest position to where the mouse is located}
  H1 := Height-HalfTH;
  W1 := Width-HalfTW;
  if IsVert then
    Result := Round(((H1-WhereY)/H1)*(Max-Min)+Min)
  else
    Result := Round((WhereX/W1)*(Max-Min)+Min);
  Result := MinInt(MaxInt(Result,Min),Max);
end;

procedure TSlideBar.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  A,B,C,D,E : Integer;
begin
  if Button <> mbLeft then exit;
  C := Position-1;
  D := Position;
  E := Position+1;
  {B is the center of the ThumbBar}
  if IsVert then B := ThumbRect.Top+HalfTH else B := ThumbRect.Left+HalfTW;
  if Dragging then
    A := NewPosition(X,Y)
  else
    if IsVert then
      if Y < B then A := E else if Y > B then A := C else A := D
    else
      if X < B then A := C else if X > B then A := E else A := D;
  A := MinInt(MaxInt(A,Min),Max);
  Dragging := False;
  Position := A;
end;

procedure TSlideBar.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  SetFocus;
  Dragging := PtInRect(ThumbRect,Point(X,Y));
  If IsVert then DragVal := Y else DragVal := X;
end;

procedure TSlideBar.MouseMove(Shift: TShiftState; X, Y: Integer);
var
  LastDragVal : Integer;
begin
  {Is the left mouse button down and dragging the thumb bar?}
  if (ssLeft in Shift) and Dragging then
    begin
      LastDragVal := DragVal;
      if IsVert then DragVal := Y else DragVal := X;
      {This test eliminates unneccesary repaints}
      if DragVal <> LastDragVal then Position := NewPosition(X,Y);
    end;
end;

procedure TSlideBar.RemoveThumbBar;
begin
  {Place the background bitmap where it was last}
  Canvas.Draw(ThumbRect.Left,ThumbRect.Top,BkgdBmp);
end;

procedure TSlideBar.DrawThumbBar;
var
  TmpBmp  : TBitMap;
  Rect1   : TRect;
begin
  { *> moved here to avoid a warning message }
  {   AFS: this is correct -
     	if there is an exception in the objects's constructor
      this you do not want to try to free the object}

  {Create a working bitmap}
  TmpBmp := TBitmap.Create;
  try
    {Define a rectangle to mark the dimensions of the thumbbar}
    Rect1 := Rect (0,0,ThumbBmp.Width,ThumbBmp.Height);
    TmpBmp.Height := ThumbBmp.Height;
    TmpBmp.Width := ThumbBmp.Width;
    {Copy the background area onto the working bitmap}
    TmpBmp.Canvas.CopyMode := cmSrcCopy;
    TmpBmp.Canvas.CopyRect(Rect1,BkgdBmp.Canvas,Rect1);
    {Copy the mask onto the working bitmap with SRCAND}
    TmpBmp.Canvas.CopyMode := cmSrcAnd;
    TmpBmp.Canvas.CopyRect(Rect1,MaskBmp.Canvas,Rect1);
    {Copy the thumbbar onto the working bitmap with SRCPAINT}
    TmpBmp.Canvas.CopyMode := cmSrcPaint;
    TmpBmp.Canvas.CopyRect(Rect1,ThumbBmp.Canvas,Rect1);
    {Now draw the thumb bar}
    Canvas.CopyRect(ThumbRect,TmpBmp.Canvas,Rect1);
  finally
    TmpBmp.Free;
  end;
end;

{ Made vertical like Win95 - zero at the top
 !!!!! in progress!!! }
procedure TSlideBar.WhereIsBar;
var
  Each          : Single;
  ThumbX,ThumbY : Integer;
begin
  {Calculate where to paint the thumb bar - store in ThumbRect}
  if IsVert then
    begin
      Each := (Height-ThumbBmp.Height)/(Max-Min);
      If Dragging then
				ThumbY := DragVal-HalfTH
      else
				ThumbY := Height-Round(Each*(Position-Min))-ThumbBmp.Height;
      ThumbY := MaxInt(0,MinInt(Height-ThumbBmp.Height,ThumbY));
      ThumbX := (Width-ThumbBmp.Width) div 2;
    end
  else
    begin
      Each := (Width-ThumbBmp.Width)/(Max-Min);
      if Dragging then
				ThumbX := DragVal-HalfTW
      else
      	ThumbX := Round(Each*(Position-Min));
      ThumbX := MaxInt(0,MinInt(Width-ThumbBmp.Width,ThumbX));
      ThumbY := (Height-ThumbBmp.Height) div 2;
    end;

  ThumbRect := Rect(ThumbX,ThumbY,ThumbX+ThumbBmp.Width,ThumbY+ThumbBmp.Height);

end; { procedure WhereIsThumb }

procedure TSlideBar.SetTLColor;
begin
  {Set the Top/Left color for the trench. Controls raised or lowered styles}
  With Canvas do
    if Style = bsLowered then Pen.Color := clGray else Pen.Color := clWhite;
end;

procedure TSlideBar.SetBRColor;
begin
  {Set the Bottom/Right color for the trench. Controls raised or lowered styles}
  With Canvas do
    if Style = bsRaised then Pen.Color := clGray else Pen.Color := clWhite;
end;

{ AFS - fixed intendation and did tick frequency }
procedure TSlideBar.DrawTrench;
var
  X1,Y1,X2,Y2 : Integer;
  Each        : Single;
  iTickLoop,TickPos : Integer;
begin
  {This procedure simply draws the slot that the thumb bar will travel through}
  {including the tick-marks. The bar itself is not drawn.}
  with Canvas do
  begin
    {Calculate the corners of the trench dependant on orientation}
    if IsVert then
    begin
      X1 := (Width div 2) - (Thickness div 2) - 1;
      X2 := X1 + Thickness + 1;
      Y1 := HalfTH;
      Y2 := Height-ThumbBmp.Height+Y1;
    end { IsVert }
    else
    begin
      X1 := HalfTW;
      X2 := Width-ThumbBmp.Width+X1;
      Y1 := (Height div 2) - (Thickness div 2) - 1;
      Y2 := Y1 + Thickness + 1;
    end; { not vert }

    Pen.Style := psSolid;
    {Set the color for the Top & Left edges}
    SetTLColor;
    MoveTo(X2,Y1);
    LineTo(X1,Y1);
    LineTo(X1,Y2);
    {Set the color for the Bottom & Right edges}
    SetBRColor;
    LineTo(X2,Y2);
    LineTo(X2,Y1-1);
    {Now do a filled black rectangle in the center if the control has focus}
    with brush do if Focused then Color := FocusColor else Color := clSilver;
    Pen.Style := psClear;
    {Draw the focus highlight}
    Rectangle(X1+1,Y1+1,X2+1,Y2+1);
    Pen.Style := psSolid;
    {Calculate spacing of tick marks}
    Each := 0;

    if Ticks then
    begin

      if (Max-Min) > 0 then
				if IsVert then
	  			Each := (Height-ThumbBmp.Height)/(Max-Min)
				else
	  			Each := (Width-ThumbBmp.Width)/(Max-Min);

      {Now draw the tick marks}
      for iTickLoop := Min to Max do
      begin
      	if (iTickLoop mod Frequency) <> 0 then
        	Continue;

        if IsVert then
          begin
            TickPos := Y2-Trunc(Each*(iTickLoop - Min)) -1;
            if iTickLoop = Max then
            	TickPos := Y1;
            SetTLColor;
            MoveTo(X1,TickPos);
            LineTo(X1-2,TickPos);

            SetBRColor;
            MoveTo(X1,TickPos+1);
            LineTo(X1-2,TickPos+1);
          end
        else
          begin
            TickPos := X1+Trunc(Each*(iTickLoop-Min));
            if iTickLoop = Max then
            	TickPos := X2-1;
            SetTLColor;
            MoveTo(TickPos,Y1);
            LineTo(TickPos,Y1-2);

            SetBRColor;
            MoveTo(TickPos+1,Y1);
            LineTo(TickPos+1,Y1-2);
          end; { Not Vert }

    	end; { for loop }

		end; { Has ticks }

	end; { with canvas }

end; { DrawTrench }

procedure TSlideBar.SaveBackground;
begin
  {This saves the background image so it can be restored later}
  BkgdBmp.Width := ThumbBmp.Width;
  BkgdBmp.Height := ThumbBmp.Height;
  BkgdBmp.Canvas.CopyRect(Rect(0,0,ThumbBmp.Width,ThumbBmp.Height),
			  Canvas,ThumbRect);
end;

procedure TSlideBar.Paint;
begin
  DrawTrench;
  WhereIsBar;
  SaveBackground;
  DrawThumbBar;
end;

{------------------------------------------------------------------------}

(*
 * About property
 *)

 procedure TAboutSlideBarProperty.Edit;
var
	msg: string;
const
	carriage_return = chr(13);
begin
	msg := 'SlideBar Component v1.1';
  AppendStr(msg, carriage_return);
  AppendStr(msg, carriage_return);
  AppendStr(msg, 'Copyright  1996 InforTech, Inc.  All Rights Reserved.');
  AppendStr(msg, carriage_return);
  AppendStr(msg, carriage_return);
  AppendStr(msg, 'Developed by:');
  AppendStr(msg, carriage_return);
  AppendStr(msg, carriage_return);
  AppendStr(msg, 'Steven Martins');
  AppendStr(msg, carriage_return);
  AppendStr(msg, 'E-Mail: steven.martins@workmail.com');
  AppendStr(msg, carriage_return);
  AppendStr(msg, 'Home Page:');
  AppendStr(msg, carriage_return);
  AppendStr(msg, 'http://www.geocities.com/SiliconValley/Park/7619/');
  AppendStr(msg, carriage_return);
  ShowMessage(msg);
end;

function TAboutSlideBarProperty.GetAttributes: TPropertyAttributes;
begin
	Result := [paMultiSelect, paDialog, paReadOnly];
end;

function TAboutSlideBarProperty.GetValue: string;
begin
	Result := 'Click on...for About box';
end;

{------------------------------------------------------------------------}
procedure Register;
begin
  RegisterComponents('InforTech', [TSlideBar]);
  {Register the about box property editor}
  RegisterPropertyEditor(TypeInfo(TAboutSlideBarProperty),
      TSlideBar, 'About', TAboutSlideBarProperty);

end;
{------------------------------------------------------------------------}

end. { Unit SlideBar }
